#include "qucircularplotcurve.h"
#include "cumacros.h"

class QuCircularPlotCurveP {
public:
    QuCircularPlotCurveP(const QString& s, QuCircularPlotCurveListener *l)
        : src(s), min(0.0), max(0.0), selected(-1), editable(false), cl(l), lpf_factor{0.0} {

    }
    QString src;
    QVector<double> x, y;
    double min, max;
    QPen pen;
    int selected;
    bool editable;
    QuCircularPlotCurveListener *cl;
    float lpf_factor;
};

QuCircularPlotCurve::QuCircularPlotCurve(const QString& src, QuCircularPlotCurveListener *l, const QPen& p)
    : d(new QuCircularPlotCurveP(src, l)) {
    d->pen = p;
}

QuCircularPlotCurve::~QuCircularPlotCurve() {
    delete d;
}

const QVector<double> &QuCircularPlotCurve::x_data() const {
    return d->x;
}

const QVector<double> &QuCircularPlotCurve::y_data() const {
    return d->y;
}

QString QuCircularPlotCurve::source() const {
    return d->src;
}

// set data on curves and always update min max even if Y autoscale is disabled
// because we always need to draw the curves accurately in our circular space
void QuCircularPlotCurve::setData(const QVector<double> &x, const QVector<double> &y) {
    if(x.size() == y.size() && x.size() > 0) {
        if(d->lpf_factor > 0 && d->y.size() == y.size()) {
            const float &f = 1.0 - d->lpf_factor;
            for(int i = 0; i < d->y.size(); i++)
                d->y[i] = d->y[i] * f + y[i] * d->lpf_factor;
        } else {
            d->y = std::move(y);
        }
        d->x = std::move(x);
        const auto [mi, ma] = std::minmax_element(d->y.begin(), d->y.end());
        bool bc = *mi != d->min || *ma != d->max;
        d->min = *mi; d->max = *ma;
        if(bc) d->cl->onCurveBoundsChanged();
    }
}

int QuCircularPlotCurve::size() const {
    return d->x.size();
}

/*! \brief low pass filter factor
 *
 *  Smooth curve transitions, adding to the current data
 *  only a small factor of the new data:
 *
 *  y[i,t] = y[i,t-1] * (1-lowPassF) + y[t, i]
 */
float QuCircularPlotCurve::lowPassF() const {
    return d->lpf_factor;
}

/*! \brief change the value of the low pass filter factor
 *
 *  @param f factor, between 0 and 1 excluded
 */
void QuCircularPlotCurve::setLowPassF(const float &f) {
    if(f > 0.0 && f < 1.0)
        d->lpf_factor = f;
    else
        perr("%s: low pass factor must be in interval (0, 1)", __PRETTY_FUNCTION__);
}

void QuCircularPlotCurve::setPen(const QPen &p) {
    d->pen = p;
}

QPen QuCircularPlotCurve::pen() const {
    return d->pen;
}

double QuCircularPlotCurve::ymax() const {
    return d->max;
}

double QuCircularPlotCurve::ymin() const {
    return d->min;
}

void QuCircularPlotCurve::minmax_update() {
    const auto [mi, ma] = std::minmax_element(d->y.begin(), d->y.end());
    d->min = *mi; d->max = *ma;
}

void QuCircularPlotCurve::select(int idx) {
    d->selected = idx;
}

void QuCircularPlotCurve::deselect() {
    d->selected = -1;
}

int QuCircularPlotCurve::selected() const {
    return d->selected;
}

bool QuCircularPlotCurve::isEditable() const {
    return d->editable;
}

void QuCircularPlotCurve::setEditable(bool e) {
    d->editable = e;
}
