#include "qucircularplotdisksector.h"

#include <QPainter>
#include <math.h>
#include <cumacros.h>
#include <qupalette.h>

class QuCircularPlotDiskSector_P {
public:
    QuCircularPlotDiskSector_P() {
        dirty = true;
    }

    bool dirty;
    QRectF rect;
};

QuCircularPlotDiskSector::QuCircularPlotDiskSector()
    : d(new QuCircularPlotDiskSector_P) {
    const QColor c1(QuPalette::instance()->value("default"));
    const QColor c2(QuPalette::instance()->value("default").darker());
    attributes.sections.names = QStringList{"S1", "S2","S3","S4","S5", "S6",
                           "S7","S8","S9","S10","S11","S12"};
    for(int i = 0; i < attributes.sections.names.size(); i++) {
        attributes.paint.sections_background <<  ((i % 2 == 0) ?  c1 :  c2);
    }
    attributes.paint.background = QuPalette::instance()->value("default");
    attributes.paint.text = QuPalette::instance()->text(QString("default"));
    attributes.paint.sections_outline = attributes.paint.background.lighter();
}

QuCircularPlotDiskSector::~QuCircularPlotDiskSector() {
    delete d;
}

void QuCircularPlotDiskSector::setSections(const QStringList &names) {
    attributes.sections.names = names;
    d->dirty = true;
}

void QuCircularPlotDiskSector::setSectionColors(const QVector<QColor> &c) {
    attributes.paint.sections_background = c;
    d->dirty = true;
}

void QuCircularPlotDiskSector::setBackgroundColor(const QColor &c) {
    attributes.paint.background = c;
}

void QuCircularPlotDiskSector::setTextDist(const float &f) {
    attributes.geom.text_distance = f;
}

QVector<QColor> QuCircularPlotDiskSector::sectionColors() const {
    return attributes.paint.sections_background;
}

QStringList QuCircularPlotDiskSector::sections() const {
    return attributes.sections.names;
}

QColor QuCircularPlotDiskSector::backgroundColor() const {
    return attributes.paint.background;
}

float QuCircularPlotDiskSector::textDist() const {
    return attributes.geom.text_distance;
}

int QuCircularPlotDiskSector::index(const QPointF &pt) {
    if(d->rect.isValid()) {
        const QPointF& c = d->rect.center();
        float alpha;
        if(pt.x() <= c.x() && pt.y() <= c.y()) {
            alpha = atan((c.y() - pt.y()) / (c.x() - pt.x()));
        }
        else if(pt.x() > c.x() && pt.y() <= c.y()) {
            alpha = M_PI / 2.0 + atan((pt.x() - c.x()) / (c.y() - pt.y()));
        }
        else if(pt.x() > c.x() && pt.y() > c.y()) {
            alpha = M_PI + atan((pt.y() - c.y()) / (pt.x() - c.x()));
        }
        else {
            alpha = 3 * M_PI / 2.0 + atan((c.x() - pt.x()) / (pt.y() - c.y()));
        }
        double a = 2 * M_PI / attributes.sections.names.size();
        for(int i = 0; i < attributes.sections.names.size(); i++) {
            if(alpha >= i * a && alpha < a * (i + 1))
                return i;
        }
    }
    return -1;
}

void QuCircularPlotDiskSector::setDrawFrom(float from) {
    attributes.geom.draw_from = from;
}

void QuCircularPlotDiskSector::setDrawTo(float to) {
    attributes.geom.draw_to = to;
}

bool QuCircularPlotDiskSector::scales() {
    return true;
}

int QuCircularPlotDiskSector::z() const {
    return -10;
}

void QuCircularPlotDiskSector::draw(QPainter *p, const QRectF &r) {
    p->save();
    QPen sp = p->pen();
    QBrush sb = p->brush();
    // align disk sector rect to the drawing area r
    const float& side = qMin(r.width(), r.height());
    QRectF rect(0.0, 0.0, side, side);
    rect.moveCenter(r.center());
    /*
     * QPainter::drawPie
     *
     * Draws a pie defined by the given rectangle, startAngle and spanAngle.
     * The pie is filled with the current brush().
     * The startAngle and spanAngle must be specified in 1/16th of a degree,
     * i.e. a full circle equals 5760 (16 * 360).
     * Positive values for the angles mean counter-clockwise while negative
     * values mean the clockwise direction. Zero degrees is at the 3 o'clock position.
     */
    const int& siz = attributes.paint.sections_background.size();
    float starta = -180 * 16;
    float a = 360.0 / siz * 16;
    p->setPen(QPen(attributes.paint.sections_outline, 0.0));
    for(int i = 0; i < siz; i++) {
        p->setBrush(attributes.paint.sections_background[i]);
        p->drawPie(rect, starta -i * a,  -a);
    }
    // restore saved brush
    p->setBrush(sb);
    // restore saved pen
    p->setPen(sp);
    // draw sections
    const QPointF& c = rect.center();
    const float R = qMin(rect.width(), rect.height()) / 2;
    float outer_radius = 0.8 * R;
    if(attributes.geom.text_distance < 0)
        attributes.geom.text_distance = (R - outer_radius) / 2.0 + outer_radius ;
    p->setPen(attributes.paint.sections_outline);
    if(attributes.paint.background.isValid() || sb != Qt::NoBrush) {
        if(attributes.geom.draw_from < 0)
            attributes.geom.draw_from = outer_radius;
        if(attributes.geom.draw_to < 0)
            attributes.geom.draw_to = R;
        if(!attributes.paint.background.isValid() && sb != Qt::NoBrush)
            p->drawEllipse(c, attributes.geom.draw_from, attributes.geom.draw_from);
        else if(attributes.paint.background.isValid()) {
            p->setBrush(attributes.paint.background);
            p->drawEllipse(c, attributes.geom.draw_from, attributes.geom.draw_from);
            p->setBrush(sb); // and restore brush
        }
    }

    QFont& font = attributes.paint.font;
    if(font == QFont())
        font = p->font();
    QFontMetrics fm(font);
    // if geometry changes, recalculate optimal font size
    if(d->rect != rect || d->dirty) {
        // maximum horizontal advance: fraction of the distance from center relative to
        // the number of elements in section
        const float hamax = R / (float) (attributes.sections.names.size() / 4.0f);
        const float fs = font.pointSizeF();
        int longest_section_idx = -1, ha = 0;
        for(int i = 0; i < attributes.sections.names.size(); i++) {
            if(ha < fm.horizontalAdvance(attributes.sections.names[i])) {
                ha = fm.horizontalAdvance(attributes.sections.names[i]);
                longest_section_idx = i;
            }
        }
        const QString smax = attributes.sections.names[longest_section_idx];
        while(fm.height() > hamax /*d->txt_dist / 8.0*/ ||
               fm.horizontalAdvance(smax) >  hamax) {
            font.setPointSizeF(font.pointSizeF()-0.20);
            fm = QFontMetrics(font);
        }
        while(fs == font.pointSizeF() && fm.height() < hamax /*d->txt_dist / 8.0*/
               && fm.horizontalAdvance(smax) < hamax) {
            font.setPointSizeF(font.pointSizeF() + 0.20);
            fm = QFontMetrics(font);
        }
        d->rect = rect;
        d->dirty = false;
    }

    a = 2 * M_PI / attributes.sections.names.size();
    QPointF pt;
    sp.setColor(attributes.paint.text);
    p->setPen(sp);
    p->setFont(font);
    for(int i = 0; i < attributes.sections.names.size(); i++) {
        pt.setX(c.x() - fm.horizontalAdvance(attributes.sections.names[i])/2.0  - attributes.geom.text_distance * cos(a/2.0 + i * a));
        pt.setY(c.y() - attributes.geom.text_distance * sin(a/2.0 + i * a));
        p->drawText(pt, attributes.sections.names[i]);
    }
    p->restore();
}

QPointF QuCircularPlotDiskSector::relativeCenter() const {
    return QPointF();
}

QSizeF QuCircularPlotDiskSector::relativeSize() const {
    return QSizeF();
}
