#include "gaugedrawable.h"
#include <math.h>
#include <QPainter>
#include <QRectF>

class GaugeDrawableP {
public:
    GaugeDrawableP() {}

    float value;
    char v_repr[32];
};

GaugeDrawable::GaugeDrawable(QObject *parent, const QPalette &palette)
    : QObject{parent}, d(new GaugeDrawableP) {
    setValue(0.0f);
    m_init_from(palette);
}

GaugeDrawable::~GaugeDrawable() {
    delete d;
}

float GaugeDrawable::value() const {
    return d->value;
}

void GaugeDrawable::setValue(float v) {
    d->value = v;
    snprintf(d->v_repr, 32, a.paint.number_format.toStdString().c_str(), v);
}

bool GaugeDrawable::scales() {
    return true;
}

int GaugeDrawable::z() const {
    return 0;
}

QSizeF GaugeDrawable::relativeSize() const {
    return a.paint.size_rel;
}

QPointF GaugeDrawable::relativeCenter() const {
    return a.paint.center_rel;
}

void GaugeDrawable::draw(QPainter *p, const QRectF &r) {
    p->save();
    p->setRenderHint(QPainter::Antialiasing, true);
    const float &s = qMin(r.width(), r.height());
    const QPointF &rc = relativeCenter();
    const QSizeF &rs = relativeSize(),
        &sqs = QSizeF(s, s); // square with min(r.width, r.height)
    QRectF area;
    if(!rc.isNull() && rs.isValid()) {
        QPointF c(r.width() * rc.x(), r.height() * rc.y());
        area.setSize(QSizeF(sqs.width() * rs.width(), sqs.height() * rs.height()));
        area.moveCenter(c);
    } else { // use full drawing area
        area = r;
    }
    if(a.paint.debug) {
        QPen bp("maroon");
        p->setPen(bp);
        p->drawText(area.x() + area.width() / 2.0, area.y(), "bounding rect");
        p->drawRect(area);
    }

    // p0, p1 degrees a0, a1 current bound
    float p0 = a.paint.start, a0 = a.bounds.min,
        a1 = a.bounds.max, p1 = a.paint.start - a.paint.span;
    const float& afactor = (a.paint.span) / (a.bounds.max - a.bounds.min);
    QColor tcol; QPen tpen;
    float spw = a.paint.scale_pen_width;
    tpen.setWidthF(spw);
    // a square line end that does not cover the end point of the line
    // otherwise with pen width >0 we get overlapping stripes, which
    // is visible if alpha channel is high enough. At this point,
    // tpen.capStyle() returns 16 (Qt::RoundCap)
    tpen.setCapStyle(Qt::FlatCap);

    float span;
    // reduce available area for drawing the indicator
    area.setTop(area.top() + spw/2.0);
    area.setLeft(area.left() + spw/2.0);
    area.setBottom(area.bottom() - spw/2.0);
    area.setRight(area.right() - spw/2.0);
    // draw thermo
    // min to min_alarm
    if(a.bounds.min_alarm < a.bounds.max_alarm
        && a0 <= a.bounds.min_alarm) {
        tcol = a.paint.gauge[gauge_dra_paint::Alarm];
        tcol.setAlpha(a.paint.scale_alpha);
        tpen.setColor(tcol);
        p->setPen(tpen);
        span = (a.bounds.min_alarm - a.bounds.min) * afactor;
        p->drawArc(area, p0 * 16, -span * 16);
        p0 = (p0 - span);
        a0 = a.bounds.min_alarm;
    }
    if(a.bounds.min_warning < a.bounds.max_warning
        && a0 <= a.bounds.min_warning) {
        tcol = a.paint.gauge[gauge_dra_paint::Warning];
        tcol.setAlpha(a.paint.scale_alpha);
        tpen.setColor(tcol);
        p->setPen(tpen);
        span = (a.bounds.min_warning - a0) * afactor;
        p->drawArc(area, p0 * 16, -span * 16);
        p0 = (p0 - span);
        a0 = a.bounds.min_warning;
    }

    // from max down a1 set to a.bounds.max

    if(a.bounds.max_alarm > a.bounds.min_alarm
        && a1 >= a.bounds.max_alarm ) {
        tcol = a.paint.gauge[gauge_dra_paint::Alarm];
        tcol.setAlpha(a.paint.scale_alpha);
        tpen.setColor(tcol);
        p->setPen(tpen);
        span = (a1 - a.bounds.max_alarm) * afactor;
        p->drawArc(area, p1 * 16, +span * 16);
        p1 = (p1 + span);
        a1 = a.bounds.max_alarm;
    }
    if(a.bounds.max_warning > a.bounds.min_warning
        && a1 >= a.bounds.max_warning ) {
        tcol = a.paint.gauge[gauge_dra_paint::Warning];
        tcol.setAlpha(a.paint.scale_alpha);
        tpen.setColor(tcol);
        p->setPen(tpen);
        span = (a1 - a.bounds.max_warning) * afactor;
        p->drawArc(area, p1 * 16, +span * 16);
        p1 = (p1 + span);
        a1 = a.bounds.max_warning;
    }
    // draw normal
    tcol = a.paint.gauge[gauge_dra_paint::Normal];
    tcol.setAlpha(a.paint.scale_alpha);
    tpen.setColor(tcol);
    p->setPen(tpen);
    span = (a1 - a0) * afactor;
    p->drawArc(area, p1 * 16, +span * 16);

    area.setTop(area.top() + spw/2.0 + a.paint.scale_gauge_spacing);
    area.setLeft(area.left() + spw/2.0 + a.paint.scale_gauge_spacing);
    area.setBottom(area.bottom() - spw/2.0 - a.paint.scale_gauge_spacing);
    area.setRight(area.right() - spw/2.0 - a.paint.scale_gauge_spacing);

    if(a.paint.debug) {
        QPen bp1("lightBlue");
        bp1.setStyle(Qt::DashLine);
        p->setPen(bp1);
        p->drawRect(area);
        p->drawText(area.topLeft(), "gauge rect");
    }

    bool error = a.bounds.max <= a.bounds.min || a.paint.error
                 || d->value < a.bounds.min || d->value > a.bounds.max;
    float start = a.paint.start;
    if(error)
        span = a.paint.span;
    else if(a.paint.baseline > a.bounds.min) { // draw from baseline
        start -= (a.paint.baseline - a.bounds.min) / (a.bounds.max - a.bounds.min)
                       * (a.paint.span);
        span =  (d->value - a.paint.baseline) / (a.bounds.max - a.bounds.min)
               * (a.paint.span);
    }
    else    {
        span = (d->value - a.bounds.min) / (a.bounds.max - a.bounds.min)
        * (a.paint.span);
    }
    QColor gauge_color(error ? QColor("gray") : a.paint.gauge[gauge_dra_paint::Normal]);
    if(!error && a.bounds.min_alarm < a.bounds.max_alarm &&
        (d->value <= a.bounds.min_alarm
         || d->value >= a.bounds.max_alarm) ) {
        gauge_color = a.paint.gauge[gauge_dra_paint::Alarm];
    }
    else if(!error && a.bounds.min_warning < a.bounds.max_warning
             && (d->value <= a.bounds.min_warning
                 || d->value >= a.bounds.max_warning) ) {
        gauge_color = a.paint.gauge[gauge_dra_paint::Warning];

    }

    QPen gpen(a.paint.number);
    gpen.setCapStyle(Qt::FlatCap);
    const float& pw = a.paint.width > 0 ? a.paint.width : r.width() / 20.0;
    gpen.setWidthF(0.0);
    p->setPen(gpen);
    QRadialGradient gra(area.center(), area.width());
    gra.setColorAt(0.0, a.paint.number);
    gra.setColorAt(0.8, gauge_color);
    p->setPen(a.paint.background);
    QBrush pieb(gra);
    p->setBrush(pieb);
    p->drawPie(area, start * 16, -span * 16);
    p->setBrush(a.paint.background);
    QRectF ar;
    ar.setLeft(area.left() + pw + spw / 2.0);
    ar.setTop(area.top() + pw  + spw / 2.0);
    ar.setBottom(area.bottom() - pw - spw / 2.0);
    ar.setRight(area.right() - pw - spw / 2.0);

    p->drawPie(ar, start * 16, -span * 16);

    p->setBrush(QBrush());

    if(a.paint.debug) {
        p->setPen(Qt::yellow);
        p->drawRect(ar);
        p->drawText(ar.topLeft(), "internal pie");
    }



    // p->setPen(gpen.color());
    // p->drawEllipse(area.center(), 5.0, 5.0);

    if(a.paint.show_number && !a.paint.error) {
        QFont f = p->font();
        p->setPen(a.paint.number);
        QFont fo = a.paint.font;
        fo.setPointSizeF(fo.pointSizeF() * a.paint.font_scale);
        if(fo != p->font()) {
            p->setFont(fo);
        }
        const QFontMetrics &fm = p->fontMetrics();
        const QPointF &tp = QPointF(ar.center().x() - fm.horizontalAdvance(d->v_repr) / 2.0,
                                    ar.center().y() + fm.height() / 2.0);
        p->drawText(tp, d->v_repr);
        if(p->font() != f)
            p->setFont(f);
    }

    if(a.paint.show_range && a.bounds.min < a.bounds.max) {
        const QFontMetrics &fm = p->fontMetrics();
        const float& dy = fm.height();
        float dx = a.paint.scale_pen_width / 2.0;
        // x : 360 = rad : 2* M_PI -> rad = x / 360.0 *
        const float &radius = area.width() / 2.0, &cx = area.center().x(),
            &cy = area.center().y();
        float x = cx + radius * cos(a.paint.start / 360.0 * 2 * M_PI);
        float y = cy - radius * sin(a.paint.start / 360.0 * 2 * M_PI);
        char n[16];
        snprintf(n, 16, a.paint.bounds_number_format.toStdString().c_str(), a.bounds.min);
        p->drawText(x - dx, y + dy, n);
        float a1 = a.paint.start - a.paint.span;
        x = cx + radius * cos(a1 / 360.0 * 2 * M_PI);
        y = cy - radius * sin(a1 / 360.0 * 2 * M_PI);
        snprintf(n, 16, a.paint.bounds_number_format.toStdString().c_str(), a.bounds.max);
        dx = -dx + fm.horizontalAdvance(n);
        p->drawText(x -dx, y + dy, n);
    }
    for(int i = 0; i < 4; i++) {
        const internal_text& t = a.paint.texts[i];
        if(t.s.length() > 0) {
            const QFontMetrics &fm = p->fontMetrics();
            const float& s = area.width() / 2.0;
            const float &cx = area.center().x(), &cy = area.center().y();
            const float& angle = /*a.paint.start -*/ a.paint.texts[i].angle * 360 /*(a.paint.span)*/;
            float x = cx + s * a.paint.texts[i].radius * cos(angle / 360.0 * 2 * M_PI) - fm.horizontalAdvance(t.s) / 2.0;
            float y = cy - s * a.paint.texts[i].radius * sin(angle / 360.0 * 2 * M_PI);
            p->drawText(x, y, t.s);
        }
    }

    p->restore();
}

void GaugeDrawable::m_init_from(const QPalette &p) {
    a.paint.background = p.color(QPalette::Window);
    a.paint.number = p.color(QPalette::WindowText);
}
